;**********************************************************************
;                                                                     *
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F84. This file contains the basic code                *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:        xxx.asm                                         *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:   Matej Baran alias FUBU					
; 				 Nemcovce 82
;				082 12 Kapusany                                                        *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: P16F84.INC                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************


    list      p=16F84             ; list directive to define processor
    #include <p16F84.inc>         ; processor specific variable definitions

    __CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON &_XT_OSC

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.

;KONSTANTY
ZERO	equ	0x7E
ONE		equ	0x0C
TWO		equ	0xB6
THREE	equ	0x9E
FOUR	equ	0xCC
FIVE	equ	0xDA
SIX		equ	0xFA	; Mapping of segments for display (PORT_B)
SEVEN	equ	0x0E
EIGHT	equ	0xFE
NINE	equ	0xCE
COLON	equ	0x01
T		equ	0xF0
BLANK	equ	0x00

MAXNTHS	equ	D'12'	; constants for timer variable count up
MAXSECS	equ	D'196'	;  variables roll over in HEX at time roll over, see variable
MAXMINS	equ	D'196'	;  explanation
MAXHRS	equ	D'244'
MINHRS	equ	D'243'
ADJMIN	equ	D'9'	; number of nths that need to be subtracted each minute
ADJHR	equ	D'34'	; nths added each hour for accurate time
ADJDAY	equ	D'3'	; nths subtracted each 1/2 day rollover

DISP1	equ	B'11111110'
DISP2	equ	B'11111101'	; Mapping of Active Display Selection (PORT_A)
DISP3	equ	B'11111011'
DISP4	equ	B'11110111'
DISPOFF	equ	H'FF'
SWITCH	equ	B'00001110'	; Activate RB1-3 for switch inputs 

;  Flag bit assignments
SEC	equ	0x00	; update time display values for sec, min, or hours
MIN	equ	0x01
HRS	equ	0x02
CHG	equ	0x03	; a change has occurred on a switch or to a potentially displayed value				 
SW1	equ	0x04	; Flag bit assignments - switches that are on = 1
SW2	equ	0x05	;  SW1 is Seconds-minutes, SW2-hours, SW3-mode
SW3	equ	0x06
SW_ON	equ	0x07	; a switch has been pressed


;***** VARIABLE DEFINITIONS
w_temp        EQU     0x0C        ; variable used for context saving 
status_temp   EQU     0x0D        ; variable used for context saving

PREMENNE	UDATA 0x20
keys	RES	1	; variable location - which keys are pressed? bit0/sw1... 
flags	RES	1	; bit flags; 0-SEC, 1-MIN, 2-HRS, 3-CHG, 4-SW1, 5-SW2, 6-SW3
display RES	1	; variable location - which display to update
digit1	RES	1	; Rightmost display value
digit2	RES	1	; Second display from right
digit3	RES	1	; Third    "       "    "
digit4	RES	1	; Fourth (and Leftmost)
;
;	timer variables start at a number that allows rollover in sync with time rollover,
;	 i.e. seconds starts at decimal 195 so that sixty 1-second increments causes 0.
sec_nth	RES	1	; seconds, fractional place
seconds	RES	1	; seconds
minutes	RES	1   ; minutes
hours	RES	1   ; hours
var	RES	1	; variable for misc math computations
count	RES	1	; loop counter variable
count2	RES	1	; 2nd loop counter for nested loops


;**********************************************************************
RESET_VECTOR      CODE    0x0000  ; processor reset vector
        goto    START             ; go to beginning of program

ISR               CODE    0x0004  ; interrupt vector location

Interrupt:

        movwf  w_temp             ; save off current W register contents
        movf   STATUS,w           ; move status register into W register
        movwf  status_temp        ; save off contents of STATUS register

;  Place ISR Here

        movf   status_temp,w      ; retrieve copy of STATUS register
        movwf  STATUS             ; restore pre-isr STATUS register contents
        swapf  w_temp,f
        swapf  w_temp,w           ; restore pre-isr W register contents
        retfie                    ; return from interrupt

MAIN_PROGRAM    CODE

START


	BANKSEL OPTION_REG
	movlw	H'03'   ; set option register, transition on clock,
    MOVWF   OPTION_REG	; Prescale RTCC, 1:16 
;
	movlw	0x00
	MOVWF	TRISA	; Set all port pins as outputs
	MOVWF	TRISB
	BANKSEL PORTB
	movlw	BLANK
	movwf	PORTB	; Blank the display
;	bcf	STATUS,PA1
;	bcf	STATUS,PA0
;
;  initialize variables
	movlw	H'01'
	movwf	TMR0	; set RTCC above zero so initial wait period occurs
	movlw	H'FE'
	movwf	display	; initializes display selected to first display.
	movlw	BLANK	; put all displays to blank, no visible segments
	movwf	digit1
	movwf	digit2
	movwf	digit3
	movwf	digit4
	movlw	MAXNTHS	; set timer variables to initial values
	movwf	sec_nth
	movlw	MAXSECS
	movwf	seconds
	movlw	MAXMINS
	movwf	minutes
	movlw	H'FF'	; hours start at 12 which is max at FF
	movwf	hours
	movlw	H'00'
	movwf	flags
;
;?  call converts for minutes and hours to initialize display vsriables
;
MAIN 
;
;  wait for RTCC to roll-over
RTCC_FILL
	movf	TMR0,0
	btfss	STATUS,Z  ; note, RTCC is left free running to not lose clock cycles on writes
	goto	RTCC_FILL
;
	incfsz	sec_nth,1  ;  add 1 to nths, n X nths = 1 sec, n is based on prescaler
	goto	TIME_DONE
	movlw	MAXNTHS
	movwf	sec_nth  ; restore sec_nths variable for next round
;
CHECK_SW
	btfss	flags,SW_ON ; if no switches press, bypass this
	goto	SET_TIME
	btfsc	flags,SW1
	goto	SET_TIME    ; if seconds display is pressed, do not change time
	movlw	MAXSECS
	movwf	seconds	    ; reset seconds to zero when setting clock
	movlw	H'7F'
	movwf	sec_nth	    ; advance second timer 1/2 second to speed time setting
	btfss	flags,SW2
	goto	HOURSET	    ; minutes do not need changing, check hours
	movlw	H'AF'
	movwf	sec_nth	    ; advances timer faster when setting minutes
	incfsz	minutes,1
	goto	HOURSET
	movlw	MAXMINS
	movwf	minutes
;
HOURSET	btfsc	flags,SW2
	goto	CHECK_TIME ; not changing hours
	incfsz	hours,1
	goto	CHECK_TIME
	movlw	MAXHRS
	movwf	hours
	goto	CHECK_TIME ; since no timing is required, go to display changes
;
SET_TIME
	bsf	flags,SEC ; seconds, if displayed, should be updated
	bsf	flags,CHG ; a flag change was made.
	incfsz	seconds,1 ;  add 1 to seconds
	goto	TIME_DONE
	movlw	MAXSECS
	movwf	seconds   ; restore seconds variable for next round
;
	bsf	flags,MIN ; minutes, if displayed, should be updated
	bsf	flags,CHG
	movlw	ADJMIN
	subwf	sec_nth,1 ; subtraction needed adjustment for each minute
	incfsz	minutes,1  ; add 1 to minutes
	goto	TIME_DONE
	movlw	MAXMINS
	movwf	minutes	  ; restore minutes variable for next hour countdown
;
	bsf	flags,HRS
	bsf	flags,CHG
	movlw	ADJHR
	addwf	sec_nth,1 ; add needed adjustment for each hour
	incfsz	hours,1	  ; add 1 to hours
	goto	TIME_DONE
	movlw	MAXHRS
	movwf	hours	  ; restore hours variable for next round
	movlw	ADJDAY
	subwf	sec_nth,1 ; subtraction adjustment for each 1/2 day rollover
;
TIME_DONE
	btfss	flags,CHG	; if no switches or potentially dislayed numbers were
 	goto	CYCLE		;  changed, then skip updating display variables
;
;
CHECK_SECONDS
;  if seconds is button was pushed and not mode display seconds
	btfss	flags,SW1
	goto	CHECK_TIME
	movlw	H'00'
	movwf	digit2	; 3rd digit variable used to store temp hex value for hours display
	movwf	digit3
	movwf	digit4
	movlw	MAXSECS
	subwf	seconds,0
	movwf	digit1  ; 1st digit variable temporarily holds hex value for seconds display
	goto	SPLIT_HEX
;
CHECK_TIME
	movlw	H'00'
	movwf	digit4	  ; zero out tens places in case there is no tens increment
	movwf	digit2
	movlw	MINHRS
	subwf	hours,0
	movwf	digit3    ; 3rd digit variable temporarily holds hex value for hours
	movlw	MAXMINS
	subwf	minutes,0
	movwf	digit1	  ; 1st digit temporarily holds hex value for minutes
;

;
;
SPLIT_HEX	;  split into two hex display variables and write
;
	movlw	H'02'
	movwf	count	; loop to convert each number - seconds - or minutes and hours

;1st time through, FSR = digit1, 2nd time FSR = digit3
	movlw	digit1	; 
	movwf	FSR	; address of digit1 into File Select Register enables POINTER
	goto	LOOP	; this loop is used to modify the minutes/seconds place
;
LOOP2	movlw	digit3
	movwf	FSR	; this loop is used to modify the hours place
;
LOOP
	movlw	D'10'
	subwf	INDF, 0x01       ; find out how many tens in number,
	btfsc	STATUS,C        ; was a borrow needed?
	goto	INCREMENT_10S   ;  if not, add 1 to tens position
	addwf	INDF, 0x01       ;  if so, do not increment tens place, add ten back on to get 1s
	goto	NEXT_DIGIT
;
INCREMENT_10S
	incf	FSR,1	  ; bump address pointed to from 1s positoion to 10s
	incf	INDF, 0x01 ; add 1 to 10s position as determined by previous subtract
	decf	FSR,1	  ; put POINTER value back to 1s place for next subtraction
	goto    LOOP	  ; go back and keep subtracting until finished
;
NEXT_DIGIT
	decfsz	count,1
	goto	LOOP2
;
CONVERT_HEX_TO_DISPLAY  ; converts hex number in digit variables to decimal display code
	movlw	digit1	
	movwf	FSR	; put the address of the first digit into the FSR to enable POINTER
	movlw	H'04'
	movwf	count	; prepare count variable to loop for all four displays
NEXT_HEX
	movf	INDF, 0x00    ; get the hex value of the current digit variable
	call	RETURN_CODE  ; call for the hex to decimal display conversion
	movwf	INDF     ; put the returned display code back into the digit variable
	incf	FSR, 0x01	     ; increment the pointer to the next digit variable address
	decfsz	count, 0x01	     ; allow only count(4) times through loop
	goto	NEXT_HEX
;
FIX_DISPLAY
	movlw	ZERO
	subwf	digit4,0
	btfss	STATUS,Z
	goto	FIX_SEC
	movlw	BLANK
	movwf	digit4

FIX_SEC	btfss	flags,SW1
	goto	CLEAR_FLAGS
	movwf	digit3	
;
CLEAR_FLAGS
	movlw	H'F0'
	andwf	flags,1	     ; clear the lower 4 flag bits to show update status
;
CYCLE
;
	movlw	DISPOFF
	movwf	PORTA	   ; Turn off LED Displays
	BANKSEL TRISB
	movlw	SWITCH
	MOVWF	TRISB	   ; Set some port B pins as switch inputs
	BANKSEL flags
	movlw	H'0F'
	andwf	flags,1	   ; reset switch flags to zero
	nop		   ; nop may not be needed, allows old outputs to bleed
	nop		   ;   off through 10k R before reading port pins
	nop
	movf	PORTB,0
	movwf	var
	btfss	var,1
	goto	SWITCH2
	bsf	flags,CHG
	bsf	flags,SW1
	bsf	flags,SW_ON
SWITCH2	btfss	var,2
	goto	SWITCH3
	bsf	flags,CHG
	bsf	flags,SW2
	bsf	flags,SW_ON
SWITCH3	btfss	var,3
	goto	SETPORT
	bsf	flags,CHG
	bsf	flags,SW3
	bsf	flags,SW_ON
;
SETPORT	
	BANKSEL TRISB
	movlw	H'00'
	MOVWF	TRISB
	BANKSEL PORTB
	movlw	BLANK
	movwf	PORTB
;
;   determine which display needs updating and cycle it on
	btfss	display,0  ; if 1st display, get 1st digit
	movf	digit4,0
	btfss	display,1  ; if 2nd display, get 2nd digit
	movf	digit3,0
	btfss	display,2  ; if 3rd display, get 3rd digit
	movf	digit2,0
	btfss	display,3  ; if 4th display, get 4th digit
	movf	digit1,0
	movwf	PORTB	   ; put the number out to display
	btfsc	sec_nth,7
	bsf	PORTB,0   ; sets colon decimal on %50 duty using highest bit
	movf	display,0  ; get display needing cycle on
	movwf	PORTA	   ; enables proper display
	movwf	display    ; returns old w if not done, new w if resetting display
	rlf	display,1  ; rotate display "on" bit to next position
	bsf	display,0  ; assures a 1 on lowest position since rotated carry is zero
	btfss	display,4  ; check if last display was already updated
	bcf	display,0  ; if it was, set display back to 1st (bit 0 set)
;
;
;
        goto    MAIN
;
RETURN_CODE
;
	addwf	PCL, 0x01
	retlw	ZERO
	retlw	ONE
	retlw	TWO
	retlw	THREE
	retlw	FOUR
	retlw	FIVE
	retlw	SIX
	retlw	SEVEN
	retlw	EIGHT
	retlw	NINE
;
;
;        org     PIC54
        goto    START

        END                       ; directive 'end of program'

